@@ -1,7 +1,7 @@
 from __future__ import absolute_import
 import sys
 
-__version__ = '0.2.2'
+__version__ = '0.2.3'
 
 # Python 2 vs 3
 PY3 = sys.version_info[0] == 3
@@ -13,9 +13,9 @@ else:
 
 # Platform specifics
 if sys.platform.startswith('win'):
-    import xlwings._xlwindows as xlplatform
+    from . import _xlwindows as xlplatform
 else:
-    import xlwings._xlmac as xlplatform
+    from . import _xlmac as xlplatform
 
 time_types = xlplatform.time_types
 
