@@ -16,6 +16,7 @@ except ImportError:
 time_types = (dt.date, dt.datetime)
 
 
+@atexit.register
 def clean_up():
     """
     Since AppleScript cannot access Excel while a Macro is running, we have to run the Python call in a
@@ -27,8 +28,6 @@ def clean_up():
     if is_excel_running():
         app('Microsoft Excel').run_VB_macro('CleanUp')
 
-atexit.register(clean_up)
-
 
 def is_file_open(fullname):
     """
@@ -265,15 +264,33 @@ def activate_chart(xl_chart):
 
 
 def autofit(range_, axis):
-    if (axis == 0 or axis == 'rows' or axis == 'r') and not range_.is_column():
-        range_.xl_range.rows.autofit()
-    elif (axis == 1 or axis == 'columns' or axis == 'c') and not range_.is_row():
-        range_.xl_range.columns.autofit()
+    address = range_.xl_range.get_address()
+    app('Microsoft Excel').screen_updating.set(False)
+    if axis == 0 or axis == 'rows' or axis == 'r':
+        range_.xl_sheet.rows[address].autofit()
+    elif axis == 1 or axis == 'columns' or axis == 'c':
+        range_.xl_sheet.columns[address].autofit()
     elif axis is None:
-        if not range_.is_row():
-            range_.xl_range.columns.autofit()
-        if not range_.is_column():
-            range_.xl_range.rows.autofit()
+        range_.xl_sheet.rows[address].autofit()
+        range_.xl_sheet.columns[address].autofit()
+    app('Microsoft Excel').screen_updating.set(True)
+
+
+def autofit_sheet(sheet, axis):
+    #TODO: combine with autofit that works on Range objects
+    num_columns = sheet.xl_sheet.count(each=kw.column)
+    num_rows = sheet.xl_sheet.count(each=kw.row)
+    xl_range = get_range_from_indices(sheet.xl_sheet, 1, 1, num_rows, num_columns)
+    address = xl_range.get_address()
+    app('Microsoft Excel').screen_updating.set(False)
+    if axis == 0 or axis == 'rows' or axis == 'r':
+        sheet.xl_sheet.rows[address].autofit()
+    elif axis == 1 or axis == 'columns' or axis == 'c':
+        sheet.xl_sheet.columns[address].autofit()
+    elif axis is None:
+        sheet.xl_sheet.rows[address].autofit()
+        sheet.xl_sheet.columns[address].autofit()
+    app('Microsoft Excel').screen_updating.set(True)
 
 
 def set_xl_workbook_current(xl_workbook):
@@ -285,4 +302,30 @@ def get_xl_workbook_current():
     try:
         return xl_workbook_current
     except NameError:
-        return None
\ No newline at end of file
+        return None
+
+
+def get_number_format(range_):
+    return range_.xl_range.number_format.get()
+
+
+def set_number_format(range_, value):
+    app('Microsoft Excel').screen_updating.set(False)
+    range_.xl_range.number_format.set(value)
+    app('Microsoft Excel').screen_updating.set(True)
+
+
+def get_address(xl_range, row_absolute, col_absolute, external):
+    return xl_range.get_address(row_absolute=row_absolute, column_absolute=col_absolute, external=external)
+
+
+def add_sheet(xl_workbook, before, after):
+    if before:
+        position = before.xl_sheet.before
+    else:
+        position = after.xl_sheet.after
+    return xl_workbook.make(new=kw.worksheet, at=position)
+
+
+def count_worksheets(xl_workbook):
+    return xl_workbook.count(each=kw.worksheet)
\ No newline at end of file
