@@ -311,15 +311,23 @@ def activate_chart(xl_chart):
 
 
 def autofit(range_, axis):
-    if (axis == 0 or axis == 'rows' or axis == 'r') and not range_.is_column():
+    if axis == 0 or axis == 'rows' or axis == 'r':
         range_.xl_range.Rows.AutoFit()
-    elif (axis == 1 or axis == 'columns' or axis == 'c') and not range_.is_row():
+    elif axis == 1 or axis == 'columns' or axis == 'c':
         range_.xl_range.Columns.AutoFit()
     elif axis is None:
-        if not range_.is_row():
-            range_.xl_range.Columns.AutoFit()
-        if not range_.is_column():
-            range_.xl_range.Rows.AutoFit()
+        range_.xl_range.Columns.AutoFit()
+        range_.xl_range.Rows.AutoFit()
+
+
+def autofit_sheet(sheet, axis):
+    if axis == 0 or axis == 'rows' or axis == 'r':
+        sheet.xl_sheet.Rows.AutoFit()
+    elif axis == 1 or axis == 'columns' or axis == 'c':
+        sheet.xl_sheet.Columns.AutoFit()
+    elif axis is None:
+        sheet.xl_sheet.Rows.AutoFit()
+        sheet.xl_sheet.Columns.AutoFit()
 
 
 def set_xl_workbook_current(xl_workbook):
@@ -331,4 +339,37 @@ def get_xl_workbook_current():
     try:
         return xl_workbook_current
     except NameError:
-        return None
\ No newline at end of file
+        return None
+
+
+def get_number_format(range_):
+    return range_.xl_range.NumberFormat
+
+
+def set_number_format(range_, value):
+    range_.xl_range.NumberFormat = value
+
+
+def get_address(xl_range, row_absolute, col_absolute, external):
+    return xl_range.GetAddress(row_absolute, col_absolute, 1, external)
+
+
+def add_sheet(xl_workbook, before, after):
+    if before:
+        return xl_workbook.Worksheets.Add(Before=before.xl_sheet)
+    else:
+        # Hack, since "After" is broken in certain environments
+        # see: http://code.activestate.com/lists/python-win32/11554/
+        count = xl_workbook.Worksheets.Count
+        new_sheet_index = after.xl_sheet.Index + 1
+        if new_sheet_index > count:
+            xl_sheet = xl_workbook.Worksheets.Add(Before=xl_workbook.Sheets(after.xl_sheet.Index))
+            xl_workbook.Worksheets(xl_workbook.Worksheets.Count).Move(Before=xl_workbook.Sheets(xl_workbook.Worksheets.Count - 1))
+            xl_workbook.Worksheets(xl_workbook.Worksheets.Count).Activate()
+        else:
+            xl_sheet = xl_workbook.Worksheets.Add(Before=xl_workbook.Sheets(after.xl_sheet.Index + 1))
+        return xl_sheet
+
+
+def count_worksheets(xl_workbook):
+    return xl_workbook.Worksheets.Count
\ No newline at end of file
