@@ -1,5 +1,5 @@
 """
-xlwings - Make Excel fly!
+xlwings - Make Excel fly with Python!
 
 Homepage and documentation: http://xlwings.org
 See also: http://zoomeranalytics.com
@@ -111,6 +111,25 @@ class Workbook(object):
         """Closes the Workbook without saving it."""
         xlplatform.close_workbook(self.xl_workbook)
 
+    @staticmethod
+    def get_xl_workbook(wkb):
+        """
+        Returns the current xl_workbook if ``wkb`` is ``None``, otherwise the ``xl_workbook`` of ``wkb``. On Windows,
+        ``xl_workbook`` is a pywin32 COM object, on Mac it's an appscript object.
+
+        Arguments
+        ---------
+        wkb : Workbook or None
+            Workbook object
+        """
+        if wkb is None and xlplatform.get_xl_workbook_current() is None:
+            raise NameError('You must first instantiate a Workbook object.')
+        elif wkb is None:
+            xl_workbook = xlplatform.get_xl_workbook_current()
+        else:
+            xl_workbook = wkb.xl_workbook
+        return xl_workbook
+
     def __repr__(self):
         return "<Workbook '{0}'>".format(self.name)
 
@@ -134,12 +153,7 @@ class Sheet(object):
     """
 
     def __init__(self, sheet, wkb=None):
-        if wkb is None and xlplatform.get_xl_workbook_current() is None:
-            raise NameError('You must first instantiate a Workbook object.')
-        elif wkb is None:
-            self.xl_workbook = xlplatform.get_xl_workbook_current()
-        else:
-            self.xl_workbook = wkb.xl_workbook
+        self.xl_workbook = Workbook.get_xl_workbook(wkb)
         self.sheet = sheet
         self.xl_sheet = xlplatform.get_xl_sheet(self.xl_workbook, self.sheet)
 
@@ -147,6 +161,30 @@ class Sheet(object):
         """Activates the sheet."""
         xlplatform.activate_sheet(self.xl_workbook, self.sheet)
 
+    def autofit(self, axis=None):
+        """
+        Autofits the width of either columns, rows or both on a whole Sheet.
+
+        Arguments
+        ---------
+        axis : string or integer, default None
+            - To autofit rows, use one of the following: 0 or ``rows`` or ``r``
+            - To autofit columns, use one of the following: 1 or ``columns`` or ``c``
+            - To autofit rows and columns, provide no arguments
+
+        Examples
+        --------
+        ::
+
+            # Autofit columns
+            Sheet('Sheet1').autofit('c')
+            # Autofit rows
+            Sheet('Sheet1').autofit('r')
+            # Autofit columns and rows
+            Range('Sheet1').autofit()
+        """
+        xlplatform.autofit_sheet(self, axis)
+
     def clear_contents(self):
         """Clears the content of the whole sheet but leaves the formatting."""
         xlplatform.clear_contents_worksheet(self.xl_workbook, self.sheet)
@@ -172,14 +210,103 @@ class Sheet(object):
     @classmethod
     def active(cls, wkb=None):
         """Returns the active Sheet. Use like so: ``Sheet.active()``"""
-        if wkb is None and xlplatform.get_xl_workbook_current() is None:
-            raise NameError('You must first instantiate a Workbook object.')
-        elif wkb is None:
-            xl_workbook = xlplatform.get_xl_workbook_current()
-        else:
-            xl_workbook = wkb.xl_workbook
+        xl_workbook = Workbook.get_xl_workbook(wkb)
         return cls(xlplatform.get_worksheet_name(xlplatform.get_active_sheet(xl_workbook)), wkb)
 
+    @classmethod
+    def add(cls, name=None, before=None, after=None, wkb=None):
+        """
+        Creates a new worksheet: the new worksheet becomes the active sheet. If neither ``before`` nor
+        ``after`` is specified, the new Sheet will be placed at the end.
+
+        Arguments
+        ---------
+        name : str, default None
+            Sheet name, defaults to Excel standard name
+
+        before : str or int, default None
+            Sheet name or index
+
+        after : str or int, default None
+            Sheet name or index
+
+        Returns
+        -------
+        Sheet object
+
+        Examples
+        --------
+
+        >>> Sheet.add()  # Place at end with default name
+        >>> Sheet.add('NewSheet', before='Sheet1')  # Include name and position
+        >>> new_sheet = Sheet.add(after=3)
+        >>> new_sheet.index
+        4
+
+        """
+        xl_workbook = Workbook.get_xl_workbook(wkb)
+
+        if before is None and after is None:
+            after = Sheet(Sheet.count())
+        elif before:
+            before = Sheet(before, wkb=wkb)
+        elif after:
+            after = Sheet(after, wkb=wkb)
+
+        if name:
+            if name in [i.name.lower() for i in Sheet.all(wkb=wkb)]:
+                raise Exception('That sheet name is already in use.')
+            else:
+                xl_sheet = xlplatform.add_sheet(xl_workbook, before, after)
+                xlplatform.set_worksheet_name(xl_sheet, name)
+                return cls(name, wkb)
+        else:
+            xl_sheet = xlplatform.add_sheet(xl_workbook, before, after)
+            return cls(xlplatform.get_worksheet_name(xl_sheet), wkb)
+
+    @staticmethod
+    def count(wkb=None):
+        """
+        Counts the number of Sheets.
+
+        Keyword Arguments
+        -----------------
+        wkb : Workbook object, default Workbook.current()
+            Defaults to the Workbook that was instantiated last or set via ``Workbook.set_current()``.
+
+        Examples
+        --------
+        >>> Sheet.count()
+        3
+        """
+        xl_workbook = Workbook.get_xl_workbook(wkb)
+        return xlplatform.count_worksheets(xl_workbook)
+
+    @staticmethod
+    def all(wkb=None):
+        """
+        Returns a list with all Sheet objects.
+
+        Keyword Arguments
+        -----------------
+        wkb : Workbook object, default Workbook.current()
+            Defaults to the Workbook that was instantiated last or set via ``Workbook.set_current()``.
+
+        Examples
+        --------
+        >>> Sheet.all()
+        [<Sheet 'Sheet1' of Workbook 'Book1'>, <Sheet 'Sheet2' of Workbook 'Book1'>]
+        >>> [i.name.lower() for i in Sheet.all()]
+        ['sheet1', 'sheet2']
+        >>> [i.autofit() for i in Sheet.all()]
+        """
+        xl_workbook = Workbook.get_xl_workbook(wkb)
+        sheet_list = []
+        for i in range(1, xlplatform.count_worksheets(xl_workbook) + 1):
+            sheet_list.append(Sheet(i, wkb=wkb))
+
+        return sheet_list
+
     def __repr__(self):
         return "<Sheet '{0}' of Workbook '{1}'>".format(self.name, xlplatform.get_workbook_name(self.xl_workbook))
 
@@ -397,8 +524,8 @@ class Range(object):
             else:
                 data = data.values[:,np.newaxis]
 
-        # NumPy array: nan have to be transformed to None, otherwise Excel shows them as 65535. This seems to be an
-        # Excel bug, see: http://visualstudiomagazine.com/articles/2008/07/01/return-double-values-in-excel.aspx
+        # NumPy array: nan have to be transformed to None, otherwise Excel shows them as 65535.
+        # See: http://visualstudiomagazine.com/articles/2008/07/01/return-double-values-in-excel.aspx
         # Also, turn into list (Python 3 can't handle arrays directly)
         if hasattr(np, 'ndarray') and isinstance(data, np.ndarray):
             try:
@@ -429,8 +556,10 @@ class Range(object):
                 # scalar np.nan need to be turned into None, otherwise Excel shows it as 65535 (same as for NumPy array)
                 if hasattr(np, 'ndarray') and np.isnan(data):
                     data = None
-            except TypeError:
-                pass  # raised if data is not a np.nan
+            except (TypeError, NotImplementedError):
+                # raised if data is not a np.nan.
+                # NumPy < 1.7.0 raises NotImplementedError, >= 1.7.0 raises TypeError
+                pass
 
         else:
             # List of List
@@ -580,6 +709,26 @@ class Range(object):
         address = xlplatform.get_current_region_address(self.xl_sheet, self.row1, self.col1)
         return Range(xlplatform.get_worksheet_name(self.xl_sheet), address, **self.kwargs)
 
+    @property
+    def number_format(self):
+        """
+        Gets and sets the number_format of a Range.
+
+        Examples
+        --------
+
+        >>> Range('A1').number_format
+        'General'
+        >>> Range('A1:C3').number_format = '0.00%'
+        >>> Range('A1:C3').number_format
+        '0.00%'
+        """
+        return xlplatform.get_number_format(self)
+
+    @number_format.setter
+    def number_format(self, value):
+        xlplatform.set_number_format(self, value)
+
     def clear(self):
         """
         Clears the content and the formatting of a Range.
@@ -608,9 +757,9 @@ class Range(object):
         ::
 
             # Autofit column A
-            Range('A:A').autofit()
+            Range('A:A').autofit('c')
             # Autofit row 1
-            Range('1:1').autofit()
+            Range('1:1').autofit('r')
             # Autofit columns and rows, taking into account Range('A1:E4')
             Range('A1:E4').autofit()
             # AutoFit columns, taking into account Range('A1:E4')
@@ -621,11 +770,62 @@ class Range(object):
         """
         xlplatform.autofit(self, axis)
 
+    def get_address(self, row_absolute=True, column_absolute=True, include_sheetname=False, external=False):
+        """
+        Returns the address of the range in the specified format.
+        
+        Arguments
+        ---------
+        row_absolute : bool, default True
+            Set to True to return the row part of the reference as an absolute reference.
+
+        column_absolute : bool, default True   
+            Set to True to return the column part of the reference as an absolute reference.
+
+        include_sheetname : bool, default False
+            Set to True to include the Sheet name in the address. Ignored if external=True.
+
+        external : bool, default False
+            Set to True to return an external reference with workbook and worksheet name.
+
+        Returns
+        -------
+        str
+
+        Examples
+        --------
+        ::
+
+            >>> Range((1,1)).get_address()
+            '$A$1'
+            >>> Range((1,1)).get_address(False, False)
+            'A1'
+            >>> Range('Sheet1', (1,1), (3,3)).get_address(True, False, True)
+            'Sheet1!A$1:C$3'
+            >>> Range('Sheet1', (1,1), (3,3)).get_address(True, False, external=True)
+            '[Workbook1]Sheet1!A$1:C$3'
+        """        
+        
+        if include_sheetname and not external:
+            # TODO: when the Workbook name contains spaces but not the Worksheet name, it will still be surrounded
+            # by '' when include_sheetname=True. Also, should probably changed to regex
+            temp_str = xlplatform.get_address(self.xl_range, row_absolute, column_absolute, True)
+
+            if temp_str.find("[") > -1:
+                results_address = temp_str[temp_str.rfind("]") + 1:]
+                if results_address.find("'") > -1:
+                    results_address = "'" + results_address
+                return results_address
+            else:
+                return temp_str
+
+        else:
+            return xlplatform.get_address(self.xl_range, row_absolute, column_absolute, external)
+
     def __repr__(self):
         return "<Range on Sheet '{0}' of Workbook '{1}'>".format(xlplatform.get_worksheet_name(self.xl_sheet),
                                                                  xlplatform.get_workbook_name(self.xl_workbook))
 
-
 class Chart(object):
     """
     A Chart object that represents an existing Excel chart can be created with the following arguments::
@@ -657,19 +857,15 @@ class Chart(object):
     >>> Range('A1').value = [['Foo1', 'Foo2'], [1, 2]]
     >>> chart = Chart.add(source_data=Range('A1').table, chart_type=ChartType.xlLine)
     >>> chart.name
-    u'Chart1'
+    'Chart1'
     >>> chart.chart_type = ChartType.xl3DArea
 
     """
     def __init__(self, *args, **kwargs):
-        # Use global Workbook if none provided
-        self.workbook = kwargs.get('wkb', None)
-        if self.workbook is None and xlplatform.get_xl_workbook_current() is None:
-            raise NameError('You must first instantiate a Workbook object.')
-        elif self.workbook is None:
-            self.xl_workbook = xlplatform.get_xl_workbook_current()
-        else:
-            self.xl_workbook = self.workbook.xl_workbook
+        # TODO: this should be doable without *args and **kwargs - same for .add()
+        # Use current Workbook if none provided
+        wkb = kwargs.get('wkb', None)
+        self.xl_workbook = Workbook.get_xl_workbook(wkb)
 
         # Arguments
         if len(args) == 1:
@@ -730,11 +926,8 @@ class Chart(object):
         wkb : Workbook object, default Workbook.current()
             Defaults to the Workbook that was instantiated last or set via ``Workbook.set_current()``.
         """
-        workbook = kwargs.get('wkb', None)
-        if workbook is None:
-            xl_workbook = xlplatform.get_xl_workbook_current()
-        else:
-            xl_workbook = workbook.xl_workbook
+        wkb = kwargs.get('wkb', None)
+        xl_workbook = Workbook.get_xl_workbook(wkb)
 
         chart_type = kwargs.get('chart_type', ChartType.xlColumnClustered)
         name = kwargs.get('name')
@@ -750,7 +943,7 @@ class Chart(object):
         else:
             name = xlplatform.get_chart_name(xl_chart)
 
-        return cls(sheet, name, wkb=workbook, chart_type=chart_type, source_data=source_data)
+        return cls(sheet, name, wkb=wkb, chart_type=chart_type, source_data=source_data)
 
     @property
     def name(self):
