@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# To run a single TestClass: nosetests -q -s test_xlwings:TestClass
+# To run a single TestClass: nosetests -q -s test_xlwings:TestRange
 
 from __future__ import unicode_literals
 import os
@@ -180,6 +180,43 @@ class TestSheet:
         cell = Range('Sheet2', 'G10').value
         assert_equal(cell, None)
 
+    def test_autofit(self):
+        Range('Sheet1', 'A1:D4').value = 'test_string'
+        Sheet('Sheet1').autofit()
+        Sheet('Sheet1').autofit(0)
+        Sheet('Sheet1').autofit(1)
+        Sheet('Sheet1').autofit('r')
+        Sheet('Sheet1').autofit('c')
+        Sheet('Sheet1').autofit('rows')
+        Sheet('Sheet1').autofit('columns')
+
+    def test_add_before(self):
+        new_sheet = Sheet.add(before='Sheet1')
+        assert_equal(Sheet(1).name, new_sheet.name)
+
+    def test_add_after(self):
+        Sheet.add(after=Sheet.count())
+        assert_equal(Sheet(Sheet.count()).name, Sheet.active().name)
+
+        Sheet.add(after=1)
+        assert_equal(Sheet(2).name, Sheet.active().name)
+
+    def test_add_default(self):
+        # TODO: test call without args properly
+        Sheet.add()
+
+    def test_add_named(self):
+        Sheet.add('test', before=1)
+        assert_equal(Sheet(1).name, 'test')
+
+    def test_count(self):
+        count = Sheet.count()
+        assert_equal(count, 3)
+
+    def test_all(self):
+        all_names = [i.name for i in Sheet.all()]
+        assert_equal(all_names, ['Sheet1', 'Sheet2', 'Sheet3'])
+
 
 class TestRange:
     def setUp(self):
@@ -500,8 +537,72 @@ class TestRange:
         result = Range('Sheet1', 'A50', atleast_2d=True, asarray=True).value
         assert_equal(np.array([[23]]), result)
 
-    def test_autofit(self):
-        pass  # TODO
+    def test_autofit_range(self):
+        # TODO: compare col/row widths before/after - not implemented yet
+        Range('Sheet1', 'A1:D4').value = 'test_string'
+        Range('Sheet1', 'A1:D4').autofit()
+        Range('Sheet1', 'A1:D4').autofit(0)
+        Range('Sheet1', 'A1:D4').autofit(1)
+        Range('Sheet1', 'A1:D4').autofit('r')
+        Range('Sheet1', 'A1:D4').autofit('c')
+        Range('Sheet1', 'A1:D4').autofit('rows')
+        Range('Sheet1', 'A1:D4').autofit('columns')
+
+    def test_autofit_col(self):
+        # TODO: compare col/row widths before/after - not implemented yet
+        Range('Sheet1', 'A1:D4').value = 'test_string'
+        Range('Sheet1', 'A:D').autofit()
+        Range('Sheet1', 'A:D').autofit(0)
+        Range('Sheet1', 'A:D').autofit(1)
+        Range('Sheet1', 'A:D').autofit('r')
+        Range('Sheet1', 'A:D').autofit('c')
+        Range('Sheet1', 'A:D').autofit('rows')
+        Range('Sheet1', 'A:D').autofit('columns')
+
+    def test_autofit_row(self):
+        # TODO: compare col/row widths before/after - not implemented yet
+        Range('Sheet1', 'A1:D4').value = 'test_string'
+        Range('Sheet1', '1:1000000').autofit()
+        Range('Sheet1', '1:1000000').autofit(0)
+        Range('Sheet1', '1:1000000').autofit(1)
+        Range('Sheet1', '1:1000000').autofit('r')
+        Range('Sheet1', '1:1000000').autofit('c')
+        Range('Sheet1', '1:1000000').autofit('rows')
+        Range('Sheet1', '1:1000000').autofit('columns')
+
+    def test_number_format_cell(self):
+        format_string = "mm/dd/yy;@"
+        Range('Sheet1', 'A1').number_format = format_string
+        result = Range('Sheet1', 'A1').number_format
+        assert_equal(format_string, result)
+
+    def test_number_format_range(self):
+        format_string = "mm/dd/yy;@"
+        Range('Sheet1', 'A1:D4').number_format = format_string
+        result = Range('Sheet1', 'A1:D4').number_format
+        assert_equal(format_string, result)
+
+    def test_get_address(self):
+        res = Range((1,1),(3,3)).get_address()
+        assert_equal(res, '$A$1:$C$3')
+
+        res = Range((1,1),(3,3)).get_address(False)
+        assert_equal(res, '$A1:$C3')
+
+        res = Range((1,1),(3,3)).get_address(True, False)
+        assert_equal(res, 'A$1:C$3')
+
+        res = Range((1,1),(3,3)).get_address(False, False)
+        assert_equal(res, 'A1:C3')
+
+        res = Range((1,1),(3,3)).get_address(include_sheetname=True)
+        assert_equal(res, 'Sheet1!$A$1:$C$3')
+
+        res = Range('Sheet2', (1,1),(3,3)).get_address(include_sheetname=True)
+        assert_equal(res, 'Sheet2!$A$1:$C$3')
+
+        res = Range((1,1),(3,3)).get_address(external=True)
+        assert_equal(res, '[test_range_1.xlsx]Sheet1!$A$1:$C$3')
 
 
 class TestChart:
